#include <math.h>
#include "param.h"

/*
 * Computes the RHS for the adaptive servo control example,
 * using " Method I " described in class.  u[0] = the REAL
 * plant input, and u[1] = the reference input r(t).  
 */

void
fx(double x[], double u[], double f[])
{
	double          yp, ym, w0, w1, w2, w3, theta0, theta1, theta2, theta3;
	double          e1, e2, epsilon1, norm, k1;

/* Plant dynamics ... P(s) = 1 / s ( s - 1 ) */
	f[0] = x[1];
	f[1] = x[1] + u[0];
	yp   = x[0];

/* Model dynamics ... M(s) = 25 / ( s + 5 ) ^ 2 */
	f[2] = x[3];
	f[3] = -25.0 * x[2] -  10.0 * x[3] + 25.0 * u[1];
	ym   = x[2];

/* State variable filters ... lambda(s) = ( s + 2 ) */
	f[4] = -2.0 * x[4] + u[0];
	f[5] = -2.0 * x[5] + yp;
	w0 = u[1];
	w1 = x[4];
	w2 = yp;
	w3 = x[5];

/* Update Law ... */
	theta0 = x[6];
	theta1 = x[7];
	theta2 = x[8];
	theta3 = x[9];
	k1 = x[10];

	e1 = yp - ym;
	e2 =  theta0 * x[11] +  theta1 * x[13] + theta2 * x[15] + theta3 * x[17] - x[19];
	epsilon1 = e1 + k1 * e2;
	norm = 1.0 + (pow(x[13], 2.0) + pow(x[15], 2.0) + pow(x[17], 2.0));

	f[6]  = -10.0 * epsilon1 * x[11] / norm;
	f[7]  = -10.0 * epsilon1 * x[13] / norm;
	f[8]  = -10.0 * epsilon1 * x[15] / norm;
	f[9]  = -10.0 * epsilon1 * x[17] / norm;
	f[10] = -10.0 * epsilon1 *  e2   / norm;

/* M[ w0 ] ... */
	f[11] = x[12];
	f[12] = -25.0 * x[11] - 10.0 * x[12] + w0;

/* M[ w1 ] ... */
	f[13] = x[14];
	f[14] = -25.0 * x[13] - 10.0 * x[14] + w1;

/* M[ w2 ] ... */
	f[15] = x[16];
	f[16] = -25.0 * x[15] - 10.0 * x[16] + w2;

/* M[ w3 ] ... */
	f[17] = x[18];
	f[18] = -25.0 * x[17] - 10.0 * x[18] + w3;

/* M[ theta0 * w0 + theta1 * w1 + theta2 * w2 + theta3 * w3 ] ... */
	f[19] = x[20];
	f[20] = -25.0 * x[19] - 10.0 * x[20] 
		+ ( theta0 * w0 +  theta1 * w1 + theta2 * w2 + theta3 * w3 );

}
