#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "param.h"
extern	void	fx(double x[], double u[], double rhs[]);

/* This procedure computes the next state x(t+delta) 	 */
/* given the present state x(t) and control u(t).	 */
/* Runge1 uses 4th order Runge-Kutta			 */
/* Runge2 uses adaptive step size 4th order Runge-Kutta. */
/* Both call a procedure fx to compute the RHS.		 */
/* delta is a global variable representing integration 	 */
/* interval for RUNGE1 or maximum integration		 */
/* interval for RUNGE2					 */


double
norm(double x1[], double x3[])
{
	int             i;
	double          epsilon, temp;

	/* Computes L-infinty distance between x1 & x3 */

	for (i = 0, epsilon = 0.0; i < N; i++) {
		temp = fabs(x1[i] - x3[i]);
		if (temp > epsilon)
			epsilon = temp;
	}
	return (epsilon);
}
/* ******************************************** */

void
Runge1(double u[], double delta, double x[])

{

	double          g0[N], g1[N], g2[N], g3[N], tempx[N];
	int             i;

	fx(x, u, g0);

	for (i = 0; i < N; i++)
		tempx[i] = x[i] + delta * g0[i] / 2.0;
	fx(tempx, u, g1);

	for (i = 0; i < N; i++)
		tempx[i] = x[i] + delta * g1[i] / 2.0;
	fx(tempx, u, g2);

	for (i = 0; i < N; i++)
		tempx[i] = x[i] + delta * g2[i];
	fx(tempx, u, g3);

	for (i = 0; i < N; i++)
		x[i] = x[i] + delta *
			(g0[i] + 2.0 * g1[i] + 2.0 * g2[i] + g3[i]) / 6.0;

}

/* ******************************************** */

void
Runge2(double u[], double report, double x[])
{

#define	steperror	2.0e-9

	double          x1[N], x2[N], x3[N];
	double          g0[N], g1[N], g2[N], g3[N], tempx[N];
	double          tau = 0.0, epsilon, delta;
	int             i;

	delta = report;

	while (1) {

		/* first compute x1(t+delta) : */

		fx(x, u, g0);
		for (i = 0; i < N; i++)
			tempx[i] = x[i] + delta * g0[i] / 2.0;
		fx(tempx, u, g1);

		for (i = 0; i < N; i++)
			tempx[i] = x[i] + delta * g1[i] / 2.0;
		fx(tempx, u, g2);

		for (i = 0; i < N; i++)
			tempx[i] = x[i] + delta * g2[i];
		fx(tempx, u, g3);

		for (i = 0; i < N; i++)
			x1[i] = x[i] + delta *
				(g0[i] + 2.0 * g1[i] + 2.0 * g2[i] + g3[i]) / 6.0;

		/* next compute x2(t+delta/2) : */

		do {
			delta /= 2.0;
			if (10000000.0 * delta < report) {
				printf("STEP SIZE ERROR!");
				break;
			}
			fx(x, u, g0);

			for (i = 0; i < N; i++)
				tempx[i] = x[i] + delta * g0[i] / 2.0;
			fx(tempx, u, g1);

			for (i = 0; i < N; i++)
				tempx[i] = x[i] + delta * g1[i] / 2.0;
			fx(tempx, u, g2);

			for (i = 0; i < N; i++)
				tempx[i] = x[i] + delta * g2[i];
			fx(tempx, u, g3);

			for (i = 0; i < N; i++)
				x2[i] = x[i] + delta * (g0[i] +
				   2.0 * g1[i] + 2.0 * g2[i] + g3[i]) / 6.0;

			/*
			 * compute x3(t+delta)  from intermediate value
			 * x2(t+delta/2):
			 */

			fx(x2, u, g0);

			for (i = 0; i < N; i++)
				tempx[i] = x2[i] + delta * g0[i] / 2.0;
			fx(tempx, u, g1);

			for (i = 0; i < N; i++)
				tempx[i] = x2[i] + delta * g1[i] / 2.0;
			fx(tempx, u, g2);

			for (i = 0; i < N; i++)
				tempx[i] = x2[i] + delta * g2[i];
			fx(tempx, u, g3);

			for (i = 0; i < N; i++)
				x3[i] = x2[i] + delta * (g0[i] +
				   2.0 * g1[i] + 2.0 * g2[i] + g3[i]) / 6.0;

			/*
			 * compare x1(t+delta) with x3(t+delta) using
			 * L-infinity norm:
			 */

			epsilon = norm(x1, x3);
			for (i = 0; i < N; i++)
				x1[i] = x2[i];

		} while (10.0 * steperror < epsilon);

		/* x3 is good enough.  Assign it to x : */

		delta *= 2.0;
		tau += delta;
		for (i = 0; i < N; i++)
			x[i] = x3[i];

		/* Can we double delta? */

		if ((epsilon < 0.2 * steperror) && (2.0 * delta <= report))
			delta *= 2.0;

		/* Are we at the report interval? */

		if (fabs(report - tau) < 1.0e-6){	/* yes we are */
			break;}

		/* Are we within delta of the report interval? */

		if ((report - tau) < delta)	/* yes we are */
			delta = report - tau;

	}

}
